#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string.h>
#include <stdint.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_video.h>
#include <time.h>
#include <vector>
#include "universe.hpp"
#include "util.hpp"
#include "textures.hpp"

static clock_t start_time, end_time;
std::vector<int> test;

static void draw_particles(universe * my_universe, std::vector<ui_particles> *my_particles)
{
    float * positions;
    int stride;
    int size;
    my_universe->get_positions(&positions, &size, &stride);
    for(int i = 0; i < size; i++)
    {
        // printf("Test\n");
        (*my_particles)[i].set_location(positions[i], positions[stride+i]);
        (*my_particles)[i].draw();
    }
}

int begin_sim(SDL_Renderer *renderer, SDL_Window *win)
{
    SDL_Rect srcR, destR;
    int num_of_particles = 10000;
    bool run = true;
    SDL_Surface *screen;
    universe my_universe = universe(num_of_particles);
    init_textures(renderer);
    std::vector<ui_particles> my_particles;
    for(int i = 0; i < num_of_particles; i++)
    {
        ui_particles temp_particle = ui_particles(Color(i%4), renderer);
        my_particles.push_back(temp_particle);
    }
    // screen = SDL_SetVideoMode(800, 800, 0, 0);
    while (run)
    {
        clock_t start_t, end_t;
        SDL_Event event;
        bool redraw = true;
        while (SDL_PollEvent(&event))
        {
            /* handle your event here */
            if (event.type == SDL_MOUSEBUTTONDOWN)
            {
                if (event.button.button == SDL_BUTTON_MIDDLE) // scroll up
                {
                    run = false;
                }
                else if ((event.button.button == SDL_BUTTON_LEFT)
                    || (event.button.button == SDL_BUTTON_RIGHT)) // scroll up
                    {
                        // bitmapSurface = SDL_LoadBMP("./sdl_sprites/red.bpm");
                        SDL_SetRenderDrawColor(renderer, 0x00, 0x00, 0x00, 0x00);
                        SDL_RenderClear(renderer);
                        my_universe.process();
                        start_time = clock();
                        draw_particles(&my_universe, &my_particles);
                        end_time = clock();

                        SDL_RenderPresent(renderer);
                        clock_t t = end_time - start_time;
                        SDL_Log("No. of clicks %ld clicks (%f seconds) to process the incoming click.\n", t, ((float)t) / CLOCKS_PER_SEC);
                }
                else
                {
                    /* Do nothing. */
                }
                
            }
            else if (event.type == SDL_QUIT)
            {
                run = false;
            }
            else if (event.type == SDL_WINDOWEVENT)
            {
                switch (event.window.event)
                {
                case SDL_WINDOWEVENT_ENTER:
                case SDL_WINDOWEVENT_SHOWN:
                case SDL_WINDOWEVENT_MOVED:
                case SDL_WINDOWEVENT_EXPOSED:
                case SDL_WINDOWEVENT_RESTORED:
                    redraw = true;
                    break;
                case SDL_WINDOWEVENT_RESIZED:
                case SDL_WINDOWEVENT_SIZE_CHANGED:
                case SDL_WINDOWEVENT_MAXIMIZED:
                    redraw = true;
                    SDL_GetWindowSize(win, &destR.w, &destR.h);
                    // ui_resize(renderer, destR.w, destR.h);
                    break;
                }
            }
            else
            {
                /* Do nothing. */
            }
        }
        if (redraw)
        {
            SDL_SetRenderDrawColor(renderer, 0x00, 0x00, 0x00, 0x00);
            SDL_RenderClear(renderer);
            start_time = clock();
            for(int i = 0; i < 100; i++)
            {
                my_universe.process();
            }
            end_time = clock();
            draw_particles(&my_universe, &my_particles);

            // ui_redraw_board(renderer);
            SDL_RenderPresent(renderer);

            clock_t t = end_time - start_time;
            SDL_Log("No. of clicks %ld clicks (%f seconds).\n",
                   t, ((float)t) / CLOCKS_PER_SEC);
        }
        /* do some other stuff here -- draw your app, etc. */
    }
    return 0;
}
