#include "custom_kernels.cuh"
#include "stdio.h"
#define BLOCK_SIZE 32

__global__ static void process_ray_collision(int size, float * norms, float * direction, float * vel, float * pos, int array_stride, float * line_vec)
{
    int index = blockIdx.x * blockDim.x + threadIdx.x;
    int stride = blockDim.x * gridDim.x;
    for (int i = index; i < size; i+= stride)
    {
        if(norms[i] < 5.0)
        {
            float speed_norm = 0.0;
            float dot_product = 0.0;
            for(int k = 0; k<3; k++)
            {
                dot_product += vel[k*array_stride+i]*line_vec[k];
            }
            for(int k = 0; k<3; k++)
            {
                float temp = (dot_product*(line_vec[k]));
                speed_norm += pow(temp, 2); 
            } 
            speed_norm = sqrt(speed_norm)*2.0;
            for(int k = 0; k<3; k++)
            {
                vel[k*array_stride+i] = vel[k*array_stride+i] - speed_norm*direction[k*size+i];
                pos[k*array_stride+i] = pos[k*array_stride+i] - direction[k*size+i]*(5.0-norms[i]);
            }
        }
    }
}


void custom_kernel_process_ray_collision(int size, float * norms, float * direction, float * vel, float * pos, int array_stride, float * line_vec)
{
    int numBlocks = (size + BLOCK_SIZE - 1) / BLOCK_SIZE;
    process_ray_collision<<<numBlocks,BLOCK_SIZE>>>(size, norms, direction, vel, pos, array_stride, line_vec);
}
